"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccountKeyRoleBased = exports.AccountKeyWeightedMultiSig = exports.AccountKeyFail = exports.AccountKeyPublic = exports.AccountKeyLegacy = exports.AccountKeyNil = exports.AccountKey = void 0;
const index_js_1 = require("../field/index.js");
const const_js_1 = require("../util/const.js");
const rlp_js_1 = require("../util/rlp.js");
// A typed AccountKey for Kaia is a FieldSet.
// https://docs.kaia.io/learn/accounts/
class AccountKey extends index_js_1.FieldSet {
}
exports.AccountKey = AccountKey;
// https://docs.kaia.io/learn/accounts/#accountkeynil-
class AccountKeyNil extends AccountKey {
    static type = const_js_1.AccountKeyType.Nil;
    static typeName = "AccountKeyNil";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
    };
    toRLP() {
        return (0, rlp_js_1.convertKeysToRLP)({ type: const_js_1.AccountKeyType.Nil });
    }
    setFieldsFromRLP(rlp) {
        if (rlp !== "0x80") {
            this.throwTypeError(`Invalid RLP string '${rlp}'`);
        }
        this.setFields({ type: const_js_1.AccountKeyType.Nil });
    }
}
exports.AccountKeyNil = AccountKeyNil;
// https://docs.kaia.io/learn/accounts/#accountkeylegacy-
class AccountKeyLegacy extends AccountKey {
    static type = const_js_1.AccountKeyType.Legacy;
    static typeName = "AccountKeyLegacy";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
    };
    toRLP() {
        return (0, rlp_js_1.convertKeysToRLP)({ type: const_js_1.AccountKeyType.Legacy });
    }
    setFieldsFromRLP(rlp) {
        if (rlp !== "0x01c0") {
            this.throwTypeError(`Invalid RLP string '${rlp}'`);
        }
        this.setFields({ type: const_js_1.AccountKeyType.Legacy });
    }
}
exports.AccountKeyLegacy = AccountKeyLegacy;
// https://docs.kaia.io/learn/accounts/#accountkeypublic-
class AccountKeyPublic extends AccountKey {
    static type = const_js_1.AccountKeyType.Public;
    static typeName = "AccountKeyPublic";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "key": index_js_1.FieldTypeCompressedPubKey,
    };
    // 0x02 + encode(CompressedPubKey)
    toRLP() {
        const rawRLP = (0, rlp_js_1.convertKeysToRLP)({ key: this.getField("key"), type: const_js_1.AccountKeyType.Public });
        return (0, rlp_js_1.GetHexlifyRLP)(const_js_1.AccountKeyType.Public, rawRLP);
    }
    setFieldsFromRLP(rlp) {
        const { type, key } = (0, rlp_js_1.decodeObjectFromRLP)(rlp);
        if (type !== this.type) {
            this.throwTypeError(`Invalid type '${type}`);
        }
        this.setFields({ type, key });
    }
}
exports.AccountKeyPublic = AccountKeyPublic;
// https://docs.kaia.io/learn/accounts/#accountkeyfail-
class AccountKeyFail extends AccountKey {
    static type = const_js_1.AccountKeyType.Fail;
    static typeName = "AccountKeyFail";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
    };
    toRLP() {
        return (0, rlp_js_1.convertKeysToRLP)({ type: const_js_1.AccountKeyType.Fail });
    }
    setFieldsFromRLP(rlp) {
        if (rlp !== "0x03c0") {
            this.throwTypeError(`Invalid RLP string '${rlp}'`);
        }
        this.setFields({ type: const_js_1.AccountKeyType.Fail });
    }
}
exports.AccountKeyFail = AccountKeyFail;
// https://docs.kaia.io/learn/accounts/#accountkeyweightedmultisig-
class AccountKeyWeightedMultiSig extends AccountKey {
    static type = const_js_1.AccountKeyType.WeightedMultiSig;
    static typeName = "AccountKeyWeightedMultiSig";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "threshold": index_js_1.FieldTypeUint32,
        "keys": index_js_1.FieldTypeWeightedPublicKeys,
    };
    // 0x04 + encode([threshold, [[weight1, compressedPubKey1], [weight2, compressedPubKey2]]])
    toRLP() {
        const [threshold, keys] = this.getFields(["threshold", "keys"]);
        const rawRLP = (0, rlp_js_1.convertKeysToRLP)({ threshold, keys, type: const_js_1.AccountKeyType.WeightedMultiSig });
        return (0, rlp_js_1.GetHexlifyRLP)(const_js_1.AccountKeyType.WeightedMultiSig, rawRLP);
    }
    setFieldsFromRLP(rlp) {
        const { type, keys, threshold } = (0, rlp_js_1.decodeObjectFromRLP)(rlp);
        if (type !== this.type) {
            this.throwTypeError(`Invalid type '${type}`);
        }
        this.setFields({ type, threshold, keys });
    }
}
exports.AccountKeyWeightedMultiSig = AccountKeyWeightedMultiSig;
// https://docs.kaia.io/learn/accounts/#accountkeyrolebased-
class AccountKeyRoleBased extends AccountKey {
    static type = const_js_1.AccountKeyType.RoleBased;
    static typeName = "AccountKeyRoleBased";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "keys": index_js_1.FieldTypeAccountKeyList,
    };
    // 0x05 + encode([key1, key2, key3])
    // = 0x05 + encode(keys)
    toRLP() {
        const inner = this.getField("keys");
        const rawRLP = (0, rlp_js_1.convertKeysToRLP)({ type: const_js_1.AccountKeyType.RoleBased, keys: inner });
        return (0, rlp_js_1.GetHexlifyRLP)(const_js_1.AccountKeyType.RoleBased, rawRLP);
    }
    setFieldsFromRLP(rlp) {
        const { type, keys } = (0, rlp_js_1.decodeObjectFromRLP)(rlp);
        if (type !== this.type) {
            this.throwTypeError(`Invalid type '${type}`);
        }
        this.setFields({ type, keys });
    }
}
exports.AccountKeyRoleBased = AccountKeyRoleBased;
