"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FieldTypeAccountKey = exports.FieldTypeAccountKeyList = exports.FieldTypeWeightedPublicKeys = exports.FieldTypeCompressedPubKey = void 0;
const index_js_1 = require("../util/index.js");
const rlp_js_1 = require("../util/rlp.js");
const helpers_js_1 = require("../util/helpers.js");
const transform_js_1 = require("../util/transform.js");
// Accepted types: A compressed (33-bytes) or uncompressed (65-bytes) public key
//                 in hex string or byte array
// Canonical type: A compressed (33-byte) public key in hex string
exports.FieldTypeCompressedPubKey = new class {
    canonicalize(value) {
        // only passed the value, input validation are applied in AccountKey.setFieldsFromRLP()
        return value;
    }
    emptyValue() { return "0x000000000000000000000000000000000000000000000000000000000000000000"; }
};
// Accepted types: An array of tuples [weight: number|string, publicKey: string]
//                 An array of objects { weight: number|string, publicKey: string }
// Canonical type: An array of hexlified tuples [weight: string, publicKey: string]
// Example canonical value:
// [
//   ["0x01", "0x02c734b50ddb229be5e929fc4aa8080ae8240a802d23d3290e5e6156ce029b110e"],
//   ["0x02", "0x0212d45f1cc56fbd6cd8fc877ab63b5092ac77db907a8a42c41dad3e98d7c64dfb"],
// ]
exports.FieldTypeWeightedPublicKeys = new class {
    canonicalize(value) {
        if (!(0, helpers_js_1.isArray)(value)) {
            throw new Error("Malformed WeightedPublicKeys");
        }
        // format to object if input is string
        return value.map((tupleOrObject) => {
            if ((0, helpers_js_1.isArray)(tupleOrObject) && tupleOrObject.length == 2) {
                return {
                    weight: tupleOrObject[0],
                    key: (0, index_js_1.getCompressedPublicKey)(tupleOrObject[1])
                };
            }
            return { weight: tupleOrObject.weight, key: (0, index_js_1.getCompressedPublicKey)(tupleOrObject.key) };
        });
    }
    emptyValue() { return []; }
};
// Accepted types: An array of AccountKeys in JS object format
//                 An array of AccountKeys in RLP format
// Canonical type: An array of AccountKeys in RLP format
// Example canonical value:
// [
//   "0x02a103e4a01407460c1c03ac0c82fd84f303a699b210c0b054f4aff72ff7dcdf01512d",
//   "0x04f84b02f848e301a103e4a01407460c1c03ac0c82fd84f303a699b210c0b054f4aff72ff7dcdf01512de301a10336f6355f5b532c3c1606f18fa2be7a16ae200c5159c8031dd25bfa389a4c9c06",
//   "0x02a102c8785266510368d9372badd4c7f4a94b692e82ba74e0b5e26b34558b0f081447",
// ]
exports.FieldTypeAccountKeyList = new class {
    canonicalize(value) {
        if (!(0, helpers_js_1.isArray)(value)) {
            throw new Error("Malformed RoleBasedKeys");
        }
        // format to object if input is string
        return value.map((elem) => {
            let key = (0, transform_js_1.clone)(elem);
            if ((0, helpers_js_1.isString)(key) && index_js_1.HexStr.isHex(key)) { // convert rlp to object
                key = (0, rlp_js_1.decodeObjectFromRLP)(key);
            }
            if (typeof key === 'object' && (0, helpers_js_1.has)(key, 'type') && (0, index_js_1.isEmbeddableAccountKeyType)(Number(key?.type))) {
                return key;
            }
            throw new Error(`AccountKeyType ${key?.type} cannot be inside an AccountKeyRoleBased`);
        });
    }
    emptyValue() { return []; }
};
// Accepted types: An AccountKey in JS object or RLP format
// Canonical type: An AccountKey in RLP format
// Example canonical value:
// "0x02a103e4a01407460c1c03ac0c82fd84f303a699b210c0b054f4aff72ff7dcdf01512d",
exports.FieldTypeAccountKey = new class {
    canonicalize(value) {
        if ((0, helpers_js_1.isString)(value) && index_js_1.HexStr.isHex(value)) { // pass RLP format
            return value;
        }
        else {
            // encode JS object format
            const rawRLP = (0, rlp_js_1.convertKeysToRLP)(value);
            return (0, rlp_js_1.GetHexlifyRLP)(value.type, rawRLP);
        }
    }
    emptyValue() { return "0x80"; }
};
