"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeFormatEVM = exports.AccountKeyType = exports.TxType = void 0;
exports.parseTxType = parseTxType;
exports.getKaikasTxType = getKaikasTxType;
exports.isKlaytnTxType = isKlaytnTxType;
exports.isBasicTxType = isBasicTxType;
exports.isFeeDelegationTxType = isFeeDelegationTxType;
exports.isPartialFeeDelegationTxType = isPartialFeeDelegationTxType;
exports.isFeePayerSigTxType = isFeePayerSigTxType;
exports.isKlaytnAccountKeyType = isKlaytnAccountKeyType;
exports.isEmbeddableAccountKeyType = isEmbeddableAccountKeyType;
const data_js_1 = require("./data.js");
const helpers_js_1 = require("./helpers.js");
const transform_js_1 = require("./transform.js");
// Klaytn Type Enumeration
var TxType;
(function (TxType) {
    // Basic
    TxType[TxType["ValueTransfer"] = 8] = "ValueTransfer";
    TxType[TxType["ValueTransferMemo"] = 16] = "ValueTransferMemo";
    TxType[TxType["AccountUpdate"] = 32] = "AccountUpdate";
    TxType[TxType["SmartContractDeploy"] = 40] = "SmartContractDeploy";
    TxType[TxType["SmartContractExecution"] = 48] = "SmartContractExecution";
    TxType[TxType["Cancel"] = 56] = "Cancel";
    // Fee Delegation
    TxType[TxType["FeeDelegatedValueTransfer"] = 9] = "FeeDelegatedValueTransfer";
    TxType[TxType["FeeDelegatedValueTransferMemo"] = 17] = "FeeDelegatedValueTransferMemo";
    TxType[TxType["FeeDelegatedAccountUpdate"] = 33] = "FeeDelegatedAccountUpdate";
    TxType[TxType["FeeDelegatedSmartContractDeploy"] = 41] = "FeeDelegatedSmartContractDeploy";
    TxType[TxType["FeeDelegatedSmartContractExecution"] = 49] = "FeeDelegatedSmartContractExecution";
    TxType[TxType["FeeDelegatedCancel"] = 57] = "FeeDelegatedCancel";
    // Partial Fee Delegation
    TxType[TxType["FeeDelegatedValueTransferWithRatio"] = 10] = "FeeDelegatedValueTransferWithRatio";
    TxType[TxType["FeeDelegatedValueTransferMemoWithRatio"] = 18] = "FeeDelegatedValueTransferMemoWithRatio";
    TxType[TxType["FeeDelegatedAccountUpdateWithRatio"] = 34] = "FeeDelegatedAccountUpdateWithRatio";
    TxType[TxType["FeeDelegatedSmartContractDeployWithRatio"] = 42] = "FeeDelegatedSmartContractDeployWithRatio";
    TxType[TxType["FeeDelegatedSmartContractExecutionWithRatio"] = 50] = "FeeDelegatedSmartContractExecutionWithRatio";
    TxType[TxType["FeeDelegatedCancelWithRatio"] = 58] = "FeeDelegatedCancelWithRatio";
})(TxType || (exports.TxType = TxType = {}));
function parseTxType(type) {
    if (type === undefined || type === null) {
        type = 0;
    }
    if (typeof type === "bigint") {
        type = Number(type);
    }
    if ((0, helpers_js_1.isNumber)(type)) {
        return type;
    }
    if ((0, helpers_js_1.isString)(type) && type.length > 0) {
        // Try the hex string, e.g. "0x08"
        if (data_js_1.HexStr.isHex(type)) {
            return data_js_1.HexStr.toNumber(type);
        }
        // Try camel case string, e.g. "ValueTransfer", "TxTypeValueTransfer"
        // or snake case string, e.g. "VALUE_TRANSFER"
        let name = type;
        if (name.startsWith("TxType")) {
            name = name.substring(6);
        }
        name = (0, transform_js_1.toUpperFirst)((0, transform_js_1.toCamelCase)(name));
        if ((0, helpers_js_1.has)(TxType, name)) {
            return (0, helpers_js_1.get)(TxType, name);
        }
    }
    throw new Error(`Unrecognized tx type '${type}'. Expected a number.'`);
}
// Convert Klaytn TxType to what Kaikas wallet (https://docs.kaikas.io/) understands.
// Pass-through undefined and non-Klaytn TxTypes.
// Convert Klaytn TxTypes to upper and snake case string (e.g. "VALUE_TRANSFER").
function getKaikasTxType(type) {
    const num = parseTxType(type);
    if (!isKlaytnTxType(num)) {
        return num;
    }
    else {
        const name = TxType[num];
        return (0, transform_js_1.toSnakeCase)(name).toUpperCase();
    }
}
// Returns true for Klaytn TxType.
function isKlaytnTxType(type) {
    return !!type && (type in TxType);
}
// Returns true for Klaytn Basic (i.e. not fee delegated) TxType.
function isBasicTxType(type) {
    return !!type && (type in TxType) && ((type & 0x3) == 0x0);
}
// Returns true for Klaytn Fee Delegated TxType.
function isFeeDelegationTxType(type) {
    return !!type && (type in TxType) && ((type & 0x3) == 0x1);
}
// Returns true for Klaytn Partial Fee Delegated (i.e. with ratio) TxType.
function isPartialFeeDelegationTxType(type) {
    return !!type && (type in TxType) && ((type & 0x3) == 0x2);
}
// Returns true for Klaytn TxType with feePayer feature (i.e. fee delegation or partial fee delegation).
function isFeePayerSigTxType(type) {
    return isFeeDelegationTxType(type) || isPartialFeeDelegationTxType(type);
}
var AccountKeyType;
(function (AccountKeyType) {
    // Account Key Type
    AccountKeyType[AccountKeyType["Nil"] = 0] = "Nil";
    AccountKeyType[AccountKeyType["Legacy"] = 1] = "Legacy";
    AccountKeyType[AccountKeyType["Public"] = 2] = "Public";
    AccountKeyType[AccountKeyType["Fail"] = 3] = "Fail";
    AccountKeyType[AccountKeyType["WeightedMultiSig"] = 4] = "WeightedMultiSig";
    AccountKeyType[AccountKeyType["RoleBased"] = 5] = "RoleBased";
})(AccountKeyType || (exports.AccountKeyType = AccountKeyType = {}));
// Returns true for Klaytn AccountKeyType.
function isKlaytnAccountKeyType(type) {
    return !!type && (type in AccountKeyType);
}
// Returns true for AccountKeyTypes that can be embedded in an AccountKeyRoleBased
// (i.e. AccountKeyNil, AccountKeyLegacy, AccountKeyPublic, AccountKeyFail, and AccountKeyWeightedMultiSig)
function isEmbeddableAccountKeyType(type) {
    return !!type && (type in AccountKeyType) && (type != AccountKeyType.RoleBased);
}
exports.CodeFormatEVM = 0x00;
