"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPublicKeyFromPrivate = getPublicKeyFromPrivate;
exports.getCompressedPublicKey = getCompressedPublicKey;
exports.getSignatureTuple = getSignatureTuple;
exports.getChainIdFromSignatureTuples = getChainIdFromSignatureTuples;
const bytes_1 = require("@ethersproject/bytes");
const elliptic_1 = __importDefault(require("elliptic"));
const data_js_1 = require("./data.js");
const helpers_js_1 = require("./helpers.js");
// ec is commonjs only, then we need default import.
const { ec } = elliptic_1.default;
const secp256k1 = new ec("secp256k1");
// Returns a 33-byte compressed public key from a private key.
function getPublicKeyFromPrivate(privateKey) {
    const key = secp256k1.keyFromPrivate(data_js_1.HexStr.stripHexPrefix(privateKey), "hex");
    return getCompressedPublicKey(key.getPublic(true, "hex"));
}
// Returns a 33-byte compressed public key from
// a compressed (33-byte), uncompressed (65-byte) public key,
// or an object { x, y } (each 32-byte).
function getCompressedPublicKey(pub) {
    if (pub instanceof Uint8Array) {
        pub = data_js_1.HexStr.from(pub);
    }
    if ((0, helpers_js_1.isString)(pub)) { // Hex string
        const hex = data_js_1.HexStr.from(data_js_1.HexStr.withHexPrefix(pub));
        if (data_js_1.HexStr.isHex(hex, 33) || data_js_1.HexStr.isHex(hex, 65)) {
            const serialized = data_js_1.HexStr.stripHexPrefix(hex);
            const pubkey = secp256k1.keyFromPublic(serialized, "hex");
            return "0x" + pubkey.getPublic(true, "hex");
        }
    }
    if ((0, helpers_js_1.isString)(pub.x) && (0, helpers_js_1.isString)(pub.y)) {
        pub.x = data_js_1.HexStr.stripHexPrefix(pub.x);
        pub.y = data_js_1.HexStr.stripHexPrefix(pub.y);
        const pubkey = secp256k1.keyFromPublic(pub);
        return "0x" + pubkey.getPublic(true, "hex");
    }
    throw new Error("Public key must be a hex string of 33 or 65 bytes, or an { x, y } object");
}
// If the sig is an array, the first element 'v' must be one of:
// - pre-EIP-155 v: {27, 28}
// - EIP-155 v: {27, 28} + chainId*8 + 2
//
// If the sig is in object form, it must have one of:
// - sig.recoveryParam: {0, 1}
// - sig.v
//   - pre-EIP-155 v: {27, 28}
//   - EIP-155 v: {27, 28} + chainId*8 + 2
//
// If the sig is bytes, it must be 64 or 65 bytes.
//
// Returns a [v,r,s] tuple composed of only strings. For example, [
//   "0x1b",
//   "0x66809fb130a6ea4ae4e823baa92573a5f1bfb4e88e64048aecfb18a2b4012b99",
//   "0x75c2c3e5f7b0a182c767137c488649cd5104a5e747371fd922d618e328e5c508",
// ]
function getSignatureTuple(sig) {
    // Pass through splitSignature() for sanity check
    let obj;
    if ((0, helpers_js_1.isArray)(sig)) {
        if (sig.length != 3) {
            throw new Error("Signature tuple must have 3 elements [v,r,s]");
        }
        const numV = data_js_1.HexStr.toNumber(sig[0]);
        obj = (0, bytes_1.splitSignature)({ v: numV, r: sig[1], s: sig[2] });
    }
    else {
        obj = (0, bytes_1.splitSignature)(sig);
    }
    // R and S must not have leading zeros
    // c.f. https://github.com/ethers-io/ethers.js/blob/v5/packages/transactions/src.ts/index.ts#L298
    return [
        data_js_1.HexStr.fromNumber(obj.v),
        data_js_1.HexStr.stripZeros(obj.r),
        data_js_1.HexStr.stripZeros(obj.s),
    ];
}
// Extract chainId from tx.txSignatures[] or tx.feePayerSignatures[].
// It works because Klaytn TxType signatures are always EIP-155.
// Returns undefined if chainId cannot be extracted. Use other methods like RPC to get chainId.
function getChainIdFromSignatureTuples(signatures) {
    if (!(0, helpers_js_1.isArray)(signatures) || signatures.length == 0) {
        return undefined;
    }
    const signature = signatures[0];
    if (!(0, helpers_js_1.isArray)(signature) || signature.length != 3) {
        return undefined;
    }
    const strV = signature[0];
    if (!data_js_1.HexStr.isHex(strV)) {
        return undefined;
    }
    // v           = 2 * chainId + {35, 36}
    // v + (v % 2) = 2 * chainId + 36
    const v = data_js_1.HexStr.toNumber(strV);
    if (v >= 35) {
        return (v + (v % 2) - 36) / 2;
    }
    else {
        return undefined;
    }
}
