"use strict";
// Unit conversion utilities
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseKaia = exports.formatKaia = exports.parseKaiaUnits = exports.formatKaiaUnits = exports.parseUnits = exports.formatUnits = void 0;
exports.formatKlayUnits = formatKlayUnits;
exports.parseKlayUnits = parseKlayUnits;
exports.formatKlay = formatKlay;
exports.parseKlay = parseKlay;
exports.fromPeb = fromPeb;
exports.toPeb = toPeb;
exports.fromKei = fromKei;
exports.toKei = toKei;
const units_1 = require("@ethersproject/units");
const helpers_js_1 = require("./helpers.js");
// All in lowercase. The ambiguity between mKLAY and MKLAY is resolved in getKlayDecimals.
const names = {
    // Klaytn units
    peb: 0,
    kpeb: 3,
    mpeb: 6,
    ston: 9,
    gpeb: 9,
    uklay: 12,
    mklay: 15,
    klay: 18,
    kklay: 21,
    Mklay: 24,
    gklay: 27,
    tklay: 30,
    // Kaia units
    kei: 0,
    gkei: 9,
    kaia: 18
};
// Returns the decimal corresponding the unitName.
// If not found, returns undefined.
function getKlayDecimals(unitName) {
    if ((0, helpers_js_1.isString)(unitName)) {
        const lower = unitName.toLowerCase();
        // mKLAY and MKLAY are different
        if (lower == "mklay" && unitName[0] == "m") {
            return 15;
        }
        else if (lower == "mklay" && unitName[0] == "M") {
            return 24;
        }
        else if (lower in names) {
            return names[lower];
        }
        else {
            return undefined;
        }
    }
    return undefined;
}
// Convert [value]peb to [unit].
function formatKlayUnits(value, unitName) {
    const decimals = getKlayDecimals(unitName);
    if (decimals) {
        // Klay units
        return (0, units_1.formatUnits)(value, decimals);
    }
    else {
        // Fallback to Eth units and raw numbers.
        // Though parseEthUnits rejects uppercase, we allow it here for consistency with getKlayDecimals.
        if ((0, helpers_js_1.isString)(unitName)) {
            unitName = unitName.toLowerCase();
        }
        return (0, units_1.formatUnits)(value, unitName);
    }
}
// Convert [value][unit] to peb.
function parseKlayUnits(value, unitName) {
    const decimals = getKlayDecimals(unitName);
    if (decimals) {
        // Klay units
        return (0, units_1.parseUnits)(value, decimals);
    }
    else {
        // Fallback to Eth units and raw numbers.
        // Though parseEthUnits rejects uppercase, we allow it here for consistency with getKlayDecimals.
        if ((0, helpers_js_1.isString)(unitName)) {
            unitName = unitName.toLowerCase();
        }
        return (0, units_1.parseUnits)(value, unitName);
    }
}
// Convert [peb]peb to KLAY
function formatKlay(peb) {
    return formatKlayUnits(peb, 18);
}
// Convert [klay]KLAY to peb
function parseKlay(klay) {
    return parseKlayUnits(klay, 18);
}
// Equivalent to web3.utils.fromWei.
// Convert [value]peb to [unit].
function fromPeb(value, unitName) {
    return formatKlayUnits(value, unitName);
}
// Equivalent to web3.utils.toWei.
// Convert [value][unit] to peb.
function toPeb(value, unitName) {
    return parseKlayUnits(value, unitName).toString();
}
// Shadow ethers functions because klay functions deals with both.
exports.formatUnits = formatKlayUnits;
exports.parseUnits = parseKlayUnits;
// Shadow kaia functions because klay functions supports same function.
exports.formatKaiaUnits = formatKlayUnits;
exports.parseKaiaUnits = parseKlayUnits;
exports.formatKaia = formatKlay;
exports.parseKaia = parseKlay;
// Equivalent to web3.utils.fromWei.
// Convert [value]kei to [unit].
function fromKei(value, unitName) {
    return formatKlayUnits(value, unitName);
}
// Equivalent to web3.utils.toWei.
// Convert [value][unit] to kei.
function toKei(value, unitName) {
    return parseKlayUnits(value, unitName).toString();
}
