import { FieldSetFactory } from "../field/index.js";
import { AccountKeyType, HexStr, getTypePrefix, isKlaytnAccountKeyType } from "../util/index.js";
import { AccountKeyFail, AccountKeyLegacy, AccountKeyNil, AccountKeyPublic, AccountKeyRoleBased, AccountKeyWeightedMultiSig } from "./accountkey.js";
export class AccountKeyFactory extends FieldSetFactory {
    constructor() {
        const requiredFields = ["type"];
        super(requiredFields);
        this.add(AccountKeyNil);
        this.add(AccountKeyLegacy);
        this.add(AccountKeyPublic);
        this.add(AccountKeyFail);
        this.add(AccountKeyWeightedMultiSig);
        this.add(AccountKeyRoleBased);
    }
    _fromObject(fields) {
        // In AccountKeyWeightedMultiSig, alias weightedPublicKeys -> keys for compatibility.
        // 'weightedPublicKeys' is used in caver and klaytn node.
        // 'keys' is used in this SDK.
        if (fields.type == AccountKeyType.WeightedMultiSig && fields.weightedPublicKeys) {
            fields.keys = fields.weightedPublicKeys;
        }
        return super.fromObject(fields);
    }
    static fromObject(fields) {
        return new AccountKeyFactory()._fromObject(fields);
    }
    _fromRLP(rlp) {
        if (rlp == "0x80") { // special case without type prefix
            return this.fromObject({ type: AccountKeyType.Nil });
        }
        const type = getTypePrefix(rlp);
        if (!isKlaytnAccountKeyType(type)) {
            throw new Error("Not a Klaytn account key");
        }
        const ctor = this.lookup(type);
        const instance = new ctor();
        instance.setFieldsFromRLP(rlp);
        return instance;
    }
    static fromRLP(rlp) {
        return new AccountKeyFactory()._fromRLP(rlp);
    }
}
export function parseAccountKey(rlp) {
    const key = AccountKeyFactory.fromRLP(rlp);
    const canonical = key.toObject();
    if (key.type == AccountKeyType.Nil || key.type == AccountKeyType.Legacy || key.type == AccountKeyType.Fail) {
        return { type: key.type };
    }
    if (key.type == AccountKeyType.Public) {
        return { type: key.type, key: canonical.key };
    }
    if (key.type == AccountKeyType.WeightedMultiSig) {
        return {
            type: key.type,
            threshold: HexStr.toNumber(canonical.threshold),
            // keys field of AccountKeyWeightedMultiSig already parse the RLP to object format
            keys: key.getField('keys'),
        };
    }
    if (key.type == AccountKeyType.RoleBased) {
        return {
            type: key.type,
            // keys field of AccountKeyRoleBased already parse the RLP to object format
            keys: key.getField('keys'),
        };
    }
    throw new Error(`Unknown AccountKeyType ${key.type}`);
}
