import { HexStr } from "../util/index.js";
export class FieldSet {
    // //////////////////////////////////////////////////////////
    // Child classes MUST override below properties and methods
    // An 1-byte type enum
    static type = -1;
    // Human readable name of the type. Appears in error messages.
    static typeName;
    // Fields declaration
    static fieldTypes;
    // End override
    // //////////////////////////////////////////////////////////
    // shortcuts for this._static.*.
    type = -1;
    typeName = "";
    fieldTypes = {};
    // Fields in their canonical forms.
    fields = {};
    constructor() {
        this.type = this._static.type;
        this.typeName = this._static.typeName;
        this.fieldTypes = this._static.fieldTypes;
    }
    // A workaround to read child class's static members.
    get _static() {
        return this.constructor;
    }
    // Fields accessors
    // Reset all fields from an object
    setFields(obj) {
        this.fields = {};
        Object.keys(this.fieldTypes).forEach((name) => {
            const fieldType = this.fieldTypes[name];
            if (obj[name] === undefined) {
                this.fields[name] = null;
                return;
            }
            try {
                this.fields[name] = fieldType.canonicalize(obj[name]);
            }
            catch (e) {
                throw new Error(`Cannot set field '${name}' to '${JSON.stringify(obj[name])}': ${e}`);
            }
        });
    }
    // Reset all fields from the array
    setFieldsFromArray(names, array) {
        this.fields = {};
        for (let i = 0; i < array.length; i++) {
            const name = names[i];
            const fieldType = this.fieldTypes[name];
            if (!fieldType) {
                throw new Error(`Unknown field '${name}' for '${this.typeName}' (type ${this.type})`);
            }
            try {
                this.fields[name] = fieldType.canonicalize(array[i]);
            }
            catch (e) {
                throw new Error(`Cannot set field '${name}' to '${JSON.stringify(array[i])}': ${e}`);
            }
        }
    }
    // Get one field
    getField(name) {
        const value = this.fields[name];
        if (value == null) {
            throw new Error(`Missing field '${name}' for '${this.typeName}' (type ${this.type})`);
        }
        return value;
    }
    // Get many fields as an array
    getFields(names) {
        return names.map((name) => this.getField(name));
    }
    toObject() {
        return this.fields;
    }
    throwTypeError(msg) {
        throw new Error(`${msg} for '${this.typeName}' (type ${HexStr.fromNumber(this.type)})`);
    }
}
export class FieldSetFactory {
    registry = {};
    requiredFields;
    constructor(requiredFields) {
        this.requiredFields = requiredFields || [];
    }
    add(cls) {
        const type = cls.type;
        const fieldTypes = cls.fieldTypes;
        if (type === -1) {
            throw new Error("Cannot register TypedFields: Missing type");
        }
        if (this.registry[type]) {
            throw new Error(`Cannot register TypedFields: type ${type} already registered`);
        }
        if (!fieldTypes) {
            throw new Error("Cannot register TypedFields: Missing fieldTypes");
        }
        for (const name of this.requiredFields) {
            if (!fieldTypes[name]) {
                throw new Error(`Cannot register TypedFields: Missing required field '${name}'`);
            }
        }
        this.registry[type] = cls;
    }
    has(type) {
        if (HexStr.isHex(type)) {
            return !!this.registry[HexStr.toNumber(type)];
        }
        else {
            return !!this.registry[type];
        }
    }
    lookup(type) {
        if (!this.has(type)) {
            throw new Error(`Unsupported type '${HexStr.fromNumber(type)}'`);
        }
        if (HexStr.isHex(type)) {
            return this.registry[HexStr.toNumber(type)];
        }
        else {
            return this.registry[type];
        }
    }
    fromObject(fields) {
        const ctor = this.lookup(fields?.type);
        const instance = new ctor();
        instance.setFields(fields);
        return instance;
    }
}
